/*
  AmpSensor.c - Library for retreiving measurement
  from LMP91000 chip via I2C. Pin number of input in 
  class definition.
  
  Version #: 1.0
  Adapted from AmpSensor.cpp
*/

#include <AmpSensor.h>
#include <common.h>
#include <math.h>
#include <I2Ccom.h>

// Global variables
const int8 chipAddress = 0b1001000;     // Chip address 
const int8 statusRegister = 0x00;     // Status register address
const int8 protectionRegister = 0x01; // Protection register address
const int8 amplifierRegister = 0x10;  // Amplifier configuration register address
const int8 referenceRegister = 0x11;  // Voltage reference register address
const int8 modeRegister = 0x12;       // Initialize mode register address

const int8 tempMode = 0b00000111;      // Temperature measurement setting (TIA amplifier on)
const int8 tempMode2 = 0b00000110;   // Temperature measurement setting (TIA amplifier off)
const int8 sleepMode = 0b00000000;     // Deep sleep operation mode
const int8 conductMode = 0b00000011;   // 3 lead amperometric cell
const int8 standbyMode = 0b00000010;   // Standby operation mode

// Gain selection values; stored in flash memory
const float gainMidpoint[] = {3069, 4943, 9899, 22130, 64714, 204923};
const float gainValues[] = {2750, 3500, 7000, 14000, 35000, 120000, 350000};
const int8 gainCommand[] = {0b00000100, 0b00001000, 0b00001100, 0b00010000, 0b00010100, 0b00011000, 0b00011100};

// CLASS TYPE CALL

void AmpSensor(int8 pin, float ref)
{
  _pin = pin;
  _vRef = ref;
  _bias = 0;
  _biasCommand = 0b00000000; // 0% positive offset bias on internal ref (1.25)/2
  //Wire.begin();
}

// PUBLIC FUNCTION CALLS

/* Sets bias to closest value to fed in value; returns real value WRT vRef; external reference voltage only
Reference Control Information (Single Byte)
Bit 7 (MSB): internal reference = 0, external reference = 1
Bit 6-5: internal zero (of reference), 00 = 20%, 01 = 50%, 10 = 67%
Bit 4: positive bias = 1, negative bias = 0
Bits 3-0: percentage bias of source reference, 0000 = 0% to 1101 = 24%, 2% per step (except 0001 = 1% & 0010 = 2%); 14 steps total
*/
float setBias(float bias)
{
   _bias = bias;
   _biasCommand = 0b10100000;            // Reset biasCommand to default state - external reference @ 50%
   
   if (bias >= 0)
     _biasCommand = _biasCommand + 16; // Set Bit 4 for positive bias
     
   int biasPercentage = (int)(100*fabs(bias/_vRef)+0.5);
   int biasCounter = 0;
   
   if (biasPercentage > 0)
   {
     while (biasCounter < biasPercentage)
     {
       if (biasCounter < 2){
         _biasCommand = _biasCommand + 1;
         biasCounter = biasCounter + 1;}
       else{
         _biasCommand = _biasCommand + 1;
         biasCounter = biasCounter + 2;}
     }
   }
   //Serial.print("Bias %: ");
   //Serial.println(biasPercentage);
   //Serial.print("Bias Command (Binary): ");
   //Serial.println(_biasCommand,BIN);
   //Serial.print("Bias Value (Ref %): ");
   //Serial.println(biasCounter);
   
   if (bias < 0)
     return ((biasCounter)/-100.0)*_vRef;
   else
     return ((biasCounter)/100.0)*_vRef;  
}

// Set new reference voltage value
void setReference(float ref)
{
  _vRef = ref;
}

// Returns raw voltage output value from AM-AFE
float rawVoltage()
{
  float cVoltage = ADC_GetResult16(_pin)*(3.3/4096);
  return cVoltage;
}

//Sets internal gain & load resistors
float internalGain(float gainResistor, int loadResistor)
{
  int8 transImpAmp = 0;
  float gainOutput = 0;
  
  // Select closest gain resistor (via halfpoint on logarithmic scale)
  if (gainResistor < gainMidpoint[0]){ // 3069 Ohm
    transImpAmp = gainCommand[0]; // B00000100
    gainOutput = gainValues[0];} // 2.75 kOhm
  else if (gainResistor < gainMidpoint[1]){  // 4943 Ohm
    transImpAmp = gainCommand[1]; // B00001000
    gainOutput = gainValues[1];} // 3.5 kOhm
  else if (gainResistor < gainMidpoint[2]){ // 9899 Ohm
    transImpAmp = gainCommand[2]; // B00001100
    gainOutput = gainValues[2];} // 7.0 kOhm
  else if (gainResistor < gainMidpoint[3]){ // 22130 Ohm
    transImpAmp = gainCommand[3]; // B00010000
    gainOutput = gainValues[3];} // 14.0 kOhm
  else if (gainResistor < gainMidpoint[4]){ // 64714 Ohm
    transImpAmp = gainCommand[4]; // B00010100
    gainOutput = gainValues[4];} // 35 kOhm
  else if (gainResistor < gainMidpoint[5]){ // 204923 Ohm
    transImpAmp = gainCommand[5]; // B00011000
    gainOutput = gainValues[5];} // 120 kOhm
  else {
    transImpAmp = gainCommand[6]; // B00011100
    gainOutput = gainValues[6];} // 350 kOhm
  
  // Select closest load matching resistor (via halfpoint)
  if (loadResistor < 22){
    transImpAmp = transImpAmp;} // 10 Ohm
  else if (loadResistor < 42){
    transImpAmp = transImpAmp + 1;} // 33 Ohm
  else if (loadResistor < 75){
    transImpAmp = transImpAmp + 2;} // 50 Ohm
  else {
    transImpAmp = transImpAmp + 3;} // 100 Ohm
  
  // Send configuration data to IC
  unlockControl();
  write8(chipAddress,amplifierRegister,transImpAmp);
  lockControl();
  return gainOutput;
}

void externalGain(int loadResistor)
{
  int8 transImpAmp = 0;
  
  // Select closest load matching resistor (via halfpoint)
  if (loadResistor < 21.5){
    transImpAmp = transImpAmp;} // 10 Ohm
  else if (loadResistor < 41.5){
    transImpAmp = transImpAmp + 1;} // 33 Ohm
  else if (loadResistor < 75){
    transImpAmp = transImpAmp + 2;} // 50 Ohm
  else {
    transImpAmp = transImpAmp + 3;} // 100 Ohm
    
  //Send configuration data to IC
  unlockControl();
  write8(chipAddress,amplifierRegister,transImpAmp);
  lockControl();
}

// Read and calculate ambient temperature on sensor IC.
// Temperature value returned from function call.
float ambientTemp()
{
   unlockControl();
   write8(chipAddress,modeRegister,tempMode);
   write8(chipAddress,referenceRegister,0b10000000);
   lockControl();
   float tempVoltage = (ADC_GetResult16(_pin)*3.3)/2048; //12 bit signed (11 bit unsigned)
   float temp = (tempVoltage-1.5622)/(-0.00816);
   return temp;
}

// Measure conductivity across sensor
float conductivity()
{
  unlockControl();
  write8(chipAddress,modeRegister,conductMode);
  write8(chipAddress,referenceRegister,_biasCommand);
  lockControl();
  float cVoltage = ADC_GetResult16(_pin)*(3.3/4096);
  return cVoltage;
}

// Read and return communication status of IC.
// 1 = Ready, 0 = Not Ready
int chipStatus()
{
  int cS = read8(chipAddress,statusRegister);
  return cS;
}

// Put IC into deep sleep mode.
void sleep()
{
  write8(chipAddress,modeRegister,sleepMode);
}

// Disable protection on registers for TI-Amp and V-Ref settings
void unlockControl()
{
  write8(chipAddress,protectionRegister,0b00000000);
}

// Enable protection on registers for TI-Amp and V-Ref settings
void lockControl()
{
  write8(chipAddress,protectionRegister,0b00000001);
}
