/*****************************************************************************
* File Name: BleApplications.c
*
* Version: 1.0
*
* Description:
* This file implements the BLE capability in the PSoC 4 BLE Lab 4.
*
* Hardware Dependency:
* CY8CKIT-042 BLE Pioneer Kit
*
******************************************************************************
* Copyright (2014), Cypress Semiconductor Corporation.
*****************************************************************************/

/*****************************************************************************
* Included headers
*****************************************************************************/
#include <BLEApplications.h>
#include <project.h>
#include <stdio.h>
#include "MCP23017.h"
#include "define.h"
#include "common.h"

#define CAPSENSE_ENABLED
#define BLE_ENABLED

#define RED_INDEX						(0)
#define GREEN_INDEX						(1)
#define BLUE_INDEX						(2)
#define INTENSITY_INDEX					(3)

#define NO_FINGER 						(0xFFFFu)

#define SLIDER_MAX_VALUE				(0x0064)

//#define TRUE							(1)
//#define FALSE							(0)
//#define ZERO							(0)

#define RGB_LED_MAX_VAL					(255)
#define RGB_LED_OFF						(255)
#define RGB_LED_ON						(0)

#define MCP0    0
#define MCP1    1
#define MCP2    2
#define PORTA   0
#define PORTB   1

/*************************Pre-processor directives****************************/
/* 'ENABLE_LOW_POWER_MODE' pre-processor directive enables the low power mode 
* handling in the firmware, ensuring low power consumption during project usage.
* To disable, comment the following #define. 
* If disabled, prevent usage of the project with coin cell */

  

/*****************************************************************************
* Static variables 
*****************************************************************************/

/* 'rgbHandle' stores RGB control data parameters */
CYBLE_GATT_HANDLE_VALUE_PAIR_T		rgbHandle;	

/*This flag is set when the Central device writes to CCC (Client Characteristic 
* Configuration) of the CapSense slider Characteristic to enable notifications */
uint8 sendCapSenseSliderNotifications = FALSE;

/* Array to store the present RGB LED control data. The 4 bytes 
* of the array represents {R, G, B, Intensity} */
uint8 RGBledData[4];

/* This flag is used by application to know whether a Central 
* device has been connected. This is updated in BLE event callback 
* function*/
uint8 deviceConnected = FALSE;

/*****************************************************************************
* Public variables 
*****************************************************************************/
bool enterHibernateFlag = false;

/*******************************************************************************
* Function Name: CustomEventHandler
********************************************************************************
* Summary:
* Call back event function to handle various events from BLE stack
*
* Parameters:
*  event:		event returned
*  eventParam:	link to value of the events returned
*
* Return:
*  void
*
*******************************************************************************/
// Also known as "appcallback" in other tutorials
void CustomEventHandler(uint32 event, void * eventParam)
{
    /* Local variable to strore the write request parameters */
	CYBLE_GATTS_WRITE_REQ_PARAM_T *wrReqParam;
    CYBLE_GAP_BD_ADDR_T localAddr; // local BT address variable
    uint8_t i;
   
    switch(event)
    {
        // GENERAL EVENTS
        case CYBLE_EVT_STACK_ON:
        	/* Start Advertisement and enter Discoverable mode*/
            //CyBle_GapSetLocalName();
			CyBle_GappStartAdvertisement(CYBLE_ADVERTISING_FAST);
            
            // Output BT address to serial debug
            CyBle_GetDeviceAddress(&localAddr);
            UART_UartPutString("\n");
            UART_UartPutString("BLE Active");
            UART_UartPutString("\n");
            char addrBLE[16];
            for(i = CYBLE_GAP_BD_ADDR_SIZE; i > 0; i--) // little endian format
            {
                sprintf(&addrBLE[2*i],"%2.2x",localAddr.bdAddr[i-1]);
            }
            UART_UartPutString(addrBLE);
            UART_UartPutString("\n");
            
			break;
        case CYBLE_EVT_TIMEOUT: 
			break;
		case CYBLE_EVT_HARDWARE_ERROR:    /* This event indicates that some internal HW error has occurred. */
            //DBG_PRINTF("Hardware Error \r\n");
            //ShowError();
			break;
        case CYBLE_EVT_HCI_STATUS:
            //DBG_PRINTF("CYBLE_EVT_HCI_STATUS: %x \r\n", *(uint8 *)eventParam);
            //ShowError();
			break;
        
        // GAP EVENTS

        // Event when advertisement starts or stops
		case CYBLE_EVT_GAPP_ADVERTISEMENT_START_STOP:
			/* Set the BLE state variable to control LED status */
            //DBG_PRINTF("CYBLE_EVT_GAPP_ADVERTISEMENT_START_STOP, state: %x\r\n", CyBle_GetState());
            if(CYBLE_STATE_DISCONNECTED == CyBle_GetState())
            {
                deviceConnected = FALSE;
                enterHibernateFlag = true;
                UART_UartPutString("Hibernating");
                UART_UartPutString("\n");
                
                SW2_ClearInterrupt();
                Wakeup_ClearPending();
                Wakeup_Start();
                CyDelay(10);
                CySysPmHibernate();
            }
			break;
            
        case CYBLE_EVT_GAP_DEVICE_DISCONNECTED:
            /* Enter hibernate mode upon disconnect
            deviceConnected = FALSE;
            enterHibernateFlag = true;
            UART_UartPutString("Hibernating");
            UART_UartPutString("\n");
            SW2_ClearInterrupt();
            Wakeup_ClearPending();
            Wakeup_Start();
            CyDelay(10);
            CySysPmHibernate();
            */
            //CyBle_GappStartAdvertisement(CYBLE_ADVERTISING_SLOW);
        break;
			
        // GATT EVENTS
            
        case CYBLE_EVT_GATT_CONNECT_IND:
            cyBle_connHandle = *(CYBLE_CONN_HANDLE_T  *)eventParam;
			/* This flag is used in application to check connection status */
			deviceConnected = TRUE;
            //CyBle_ProcessEvents();
			break;
        
        case CYBLE_EVT_GATT_DISCONNECT_IND:
			/* Update deviceConnected flag*/
			deviceConnected = FALSE;
            //CyBle_GappStartAdvertisement(CYBLE_ADVERTISING_FAST);
            isConnectionUpdateRequested = TRUE;
			break;
            
        case CYBLE_EVT_GATTS_WRITE_REQ: 							
            /* This event is received when Central device sends a Write command 
             * on an Attribute. */
            wrReqParam = (CYBLE_GATTS_WRITE_REQ_PARAM_T *) eventParam;
            
            /* Extract the attribute handle for the RGB LED characteristic 
             * from the custom service data structure.
             */
             
            /* if (attributeHandle == RGB LED Characteristic Handle) */
            /*if(wrReqParam->handleValPair.attrHandle == cyBle_customs[RGB_LED_SERVICE_INDEX].\
								                        customServiceInfo[RGB_LED_CHAR_INDEX].\
                                                        customServiceCharHandle)
            */
            //{
                /* Extract the value of the attribute from the handle-value 
                 * pair database. Index of byte array*/
            //    RGBledData[RED_INDEX] = wrReqParam->handleValPair.value.val[RED_INDEX];
            //    RGBledData[GREEN_INDEX] = wrReqParam->handleValPair.value.val[GREEN_INDEX];
            //    RGBledData[BLUE_INDEX] = wrReqParam->handleValPair.value.val[BLUE_INDEX];
            //    RGBledData[INTENSITY_INDEX] = wrReqParam->handleValPair.value.val[INTENSITY_INDEX];
                
                /* Update the PrISM components and the attribute for RGB LED read 
                 * characteristics */
            //    UpdateRGBled();
            //}
            
             /* if (attributeHandle == RGB LED Characteristic Handle) */
            /*if(wrReqParam->handleValPair.attrHandle == cyBle_customs[NECTAR_SERVICE_INDEX].\
								                        customServiceInfo[MEASUREMENT_CHAR_INDEX].\
                                                        customServiceCharHandle)
            */
            //{
                /* Extract the value of the attribute from the handle-value 
                 * pair database. Index of byte array*/
            //    RGBledData[RED_INDEX] = wrReqParam->handleValPair.value.val[RED_INDEX];
            //    RGBledData[GREEN_INDEX] = wrReqParam->handleValPair.value.val[GREEN_INDEX];
            //    RGBledData[BLUE_INDEX] = wrReqParam->handleValPair.value.val[BLUE_INDEX];
            //    RGBledData[INTENSITY_INDEX] = wrReqParam->handleValPair.value.val[INTENSITY_INDEX];
                
                /* Update the PrISM components and the attribute for RGB LED read 
                 * characteristics */
            //    UpdateRGBled();
            //}
           
			/* Send the response to the write request received. */
			CyBle_GattsWriteRsp(cyBle_connHandle);
            break;
			
        case CYBLE_EVT_L2CAP_CONN_PARAM_UPDATE_RSP:
				/* If L2CAP connection parameter update response received, reset application flag */
            	isConnectionUpdateRequested = FALSE;
			break;

        default:

       	 	break;
    }
}


/*******************************************************************************
* Function Name: UpdateRGBled
********************************************************************************
* Summary:
* Receive the new RGB data and modify PrISM parameters. Also, update the
* read characteristic handle so that the next read from the BLE central device
* gives present RGB color and intensity data.
*******************************************************************************/
void UpdateRGBled(void)
{
	/* Local variables to calculate the color components from RGB received data*/
	//uint8 debug_red;
	//uint8 debug_green;
	//uint8 debug_blue;
	//uint8 intensity_divide_value = RGBledData[INTENSITY_INDEX];
	
	//debug_red = (uint8)(((uint16)RGBledData[RED_INDEX] * intensity_divide_value) / 255);
	//debug_green = (uint8)(((uint16)RGBledData[GREEN_INDEX] * intensity_divide_value) / 255);
	//debug_blue = (uint8)(((uint16)RGBledData[BLUE_INDEX] * intensity_divide_value) / 255);
	
	/* Update the density value of the PrISM module for color control*/
    //if (RGBledData[0] < 125)
    //    pinWrite(MCP0,0,LOW);
    //else
    //    pinWrite(MCP0,0,HIGH);
        
	//PRS_1_WritePulse0(RGB_LED_MAX_VAL - debug_red);
   // PRS_1_WritePulse1(RGB_LED_MAX_VAL - debug_green);
    //PRS_2_WritePulse0(RGB_LED_MAX_VAL - debug_blue);
	
	/* Update RGB control handle with new values */
	//rgbHandle.attrHandle = RGB_LED_CHAR_HANDLE;
	//rgbHandle.value.val = RGBledData;
	//rgbHandle.value.len = sizeof(RGBledData);
	//rgbHandle.value.actualLen = sizeof(RGBledData);
	
	/* Send updated RGB control handle as attribute for read by central device */
	//CyBle_GattsWriteAttributeValue(&rgbHandle, FALSE, &cyBle_connHandle, FALSE);
        //CyBle_GattcWriteCharacteristicValue(
}


/* [] END OF FILE */
