/***************************************************************************
  This is a library for the BME280 humidity, temperature & pressure sensor

  Designed specifically to work with the Adafruit BME280 Breakout
  ----> http://www.adafruit.com/products/2650

  These sensors use I2C or SPI to communicate, 2 or 4 pins are required
  to interface.

 ***************************************************************************/
#include <I2Ccom.h>
#include <BME280.h>

/***************************************************************************
 PRIVATE FUNCTIONS
 ***************************************************************************/

int BME280_begin(void) 
{

  if (read8(BME280_ADDRESS, BME280_REGISTER_CHIPID) != 0x60)
    return 0;

  readCoefficients();

  //Set before CONTROL_meas (DS 5.4.3)
  write8(BME280_ADDRESS, BME280_REGISTER_CONTROLHUMID, 0x05); //16x oversampling 
  write8(BME280_ADDRESS, BME280_REGISTER_CONTROL, 0xB7); // 16x ovesampling, normal mode
  return 1;
}


/**************************************************************************/
/*!
    @brief  Reads the factory-set coefficients
*/
/**************************************************************************/
void readCoefficients(void)
{
    _bme280_calib.dig_T1 = read16_LE(BME280_ADDRESS, BME280_REGISTER_DIG_T1);
    _bme280_calib.dig_T2 = read16S_LE(BME280_ADDRESS, BME280_REGISTER_DIG_T2);
    _bme280_calib.dig_T3 = read16S_LE(BME280_ADDRESS, BME280_REGISTER_DIG_T3);

    _bme280_calib.dig_P1 = read16_LE(BME280_ADDRESS, BME280_REGISTER_DIG_P1);
    _bme280_calib.dig_P2 = read16S_LE(BME280_ADDRESS, BME280_REGISTER_DIG_P2);
    _bme280_calib.dig_P3 = read16S_LE(BME280_ADDRESS, BME280_REGISTER_DIG_P3);
    _bme280_calib.dig_P4 = read16S_LE(BME280_ADDRESS, BME280_REGISTER_DIG_P4);
    _bme280_calib.dig_P5 = read16S_LE(BME280_ADDRESS, BME280_REGISTER_DIG_P5);
    _bme280_calib.dig_P6 = read16S_LE(BME280_ADDRESS, BME280_REGISTER_DIG_P6);
    _bme280_calib.dig_P7 = read16S_LE(BME280_ADDRESS, BME280_REGISTER_DIG_P7);
    _bme280_calib.dig_P8 = read16S_LE(BME280_ADDRESS, BME280_REGISTER_DIG_P8);
    _bme280_calib.dig_P9 = read16S_LE(BME280_ADDRESS, BME280_REGISTER_DIG_P9);

    _bme280_calib.dig_H1 = read8(BME280_ADDRESS, BME280_REGISTER_DIG_H1);
    _bme280_calib.dig_H2 = read16S_LE(BME280_ADDRESS, BME280_REGISTER_DIG_H2);
    _bme280_calib.dig_H3 = read8(BME280_ADDRESS, BME280_REGISTER_DIG_H3);
    //_bme280_calib.dig_H4 = (read8(BME280_ADDRESS, BME280_REGISTER_DIG_H4_MSB) << 4) | (read8(BME280_ADDRESS, BME280_REGISTER_DIG_H4) & 0x0F);
    //_bme280_calib.dig_H5 = (read8(BME280_ADDRESS, BME280_REGISTER_DIG_H5_MSB) << 4) | ((read8(BME280_ADDRESS, BME280_REGISTER_DIG_H4) >> 4) & 0x0F);
    //_bme280_calib.dig_H6 = read8(BME280_ADDRESS, BME280_REGISTER_DIG_H6);
    _bme280_calib.dig_H4 = (read8(BME280_ADDRESS,BME280_REGISTER_DIG_H4) << 4) | (read8(BME280_ADDRESS,BME280_REGISTER_DIG_H4+1) & 0xF);
    _bme280_calib.dig_H5 = (read8(BME280_ADDRESS,BME280_REGISTER_DIG_H5_MSB+1) << 4) | (read8(BME280_ADDRESS,BME280_REGISTER_DIG_H5_MSB) >> 4);
    _bme280_calib.dig_H6 = (int8_t)read8(BME280_ADDRESS,BME280_REGISTER_DIG_H6);
}

/**************************************************************************/
/*!

*/
/**************************************************************************/
float readTemperature(void)
{
  int32_t var1, var2;

  int32_t adc_T = read24(BME280_ADDRESS,BME280_REGISTER_TEMPDATA);
  adc_T >>= 4;

  var1  = ((((adc_T>>3) - ((int32_t)_bme280_calib.dig_T1 <<1))) *
	   ((int32_t)_bme280_calib.dig_T2)) >> 11;

  var2  = (((((adc_T>>4) - ((int32_t)_bme280_calib.dig_T1)) *
	     ((adc_T>>4) - ((int32_t)_bme280_calib.dig_T1))) >> 12) *
	   ((int32_t)_bme280_calib.dig_T3)) >> 14;

  t_fine = var1 + var2;

  float T  = (t_fine * 5 + 128) >> 8;
  return T/100;
}

/**************************************************************************/
/*!

*/
/**************************************************************************/
float readPressure(void) {
  int64_t var1, var2, p;

  readTemperature(); // must be done first to get t_fine

  int32_t adc_P = read24(BME280_ADDRESS,BME280_REGISTER_PRESSUREDATA);
  adc_P >>= 4;

  var1 = ((int64_t)t_fine) - 128000;
  var2 = var1 * var1 * (int64_t)_bme280_calib.dig_P6;
  var2 = var2 + ((var1*(int64_t)_bme280_calib.dig_P5)<<17);
  var2 = var2 + (((int64_t)_bme280_calib.dig_P4)<<35);
  var1 = ((var1 * var1 * (int64_t)_bme280_calib.dig_P3)>>8) +
    ((var1 * (int64_t)_bme280_calib.dig_P2)<<12);
  var1 = (((((int64_t)1)<<47)+var1))*((int64_t)_bme280_calib.dig_P1)>>33;

  if (var1 == 0) {
    return 0;  // avoid exception caused by division by zero
  }
  p = 1048576 - adc_P;
  p = (((p<<31) - var2)*3125) / var1;
  var1 = (((int64_t)_bme280_calib.dig_P9) * (p>>13) * (p>>13)) >> 25;
  var2 = (((int64_t)_bme280_calib.dig_P8) * p) >> 19;

  p = ((p + var1 + var2) >> 8) + (((int64_t)_bme280_calib.dig_P7)<<4);
  return (float)p/256;
}


/**************************************************************************/
/*!

*/
/**************************************************************************/
float readHumidity(void) {

  readTemperature(); // must be done first to get t_fine

  int32_t adc_H = read16(BME280_ADDRESS,BME280_REGISTER_HUMIDDATA);

  int32_t v_x1_u32r;

  v_x1_u32r = (t_fine - ((int32_t)76800));

  v_x1_u32r = (((((adc_H << 14) - (((int32_t)_bme280_calib.dig_H4) << 20) -
		  (((int32_t)_bme280_calib.dig_H5) * v_x1_u32r)) + ((int32_t)16384)) >> 15) *
	       (((((((v_x1_u32r * ((int32_t)_bme280_calib.dig_H6)) >> 10) *
		    (((v_x1_u32r * ((int32_t)_bme280_calib.dig_H3)) >> 11) + ((int32_t)32768))) >> 10) +
		  ((int32_t)2097152)) * ((int32_t)_bme280_calib.dig_H2) + 8192) >> 14));

  v_x1_u32r = (v_x1_u32r - (((((v_x1_u32r >> 15) * (v_x1_u32r >> 15)) >> 7) *
			     ((int32_t)_bme280_calib.dig_H1)) >> 4));

  v_x1_u32r = (v_x1_u32r < 0) ? 0 : v_x1_u32r;
  v_x1_u32r = (v_x1_u32r > 419430400) ? 419430400 : v_x1_u32r;
  float h = (v_x1_u32r>>12);
  return  h / 1024.0;
}

/**************************************************************************/
/*!
    Calculates the altitude (in meters) from the specified atmospheric
    pressure (in hPa), and sea-level pressure (in hPa).

    @param  seaLevel      Sea-level pressure in hPa
    @param  atmospheric   Atmospheric pressure in hPa
*/
/**************************************************************************/
float readAltitude(float seaLevel)
{
  // Equation taken from BMP180 datasheet (page 16):
  //  http://www.adafruit.com/datasheets/BST-BMP180-DS000-09.pdf

  // Note that using the equation from wikipedia can give bad results
  // at high altitude.  See this thread for more information:
  //  http://forums.adafruit.com/viewtopic.php?f=22&t=58064

  float atmospheric = readPressure() / 100.0F;
  return 44330.0 * (1.0 - pow(atmospheric / seaLevel, 0.1903));
}
