/*******************************************************************************
* File Name: ADCBuffer.c
* Version 1.10
*
* Description:
*  This file provides the source code to the API for the Opamp (Analog Buffer)
*  Component.
*
*
********************************************************************************
* Copyright 2013-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "ADCBuffer.h"

uint8 ADCBuffer_initVar = 0u; /* Defines if component was initialized */
static uint32 ADCBuffer_internalPower = 0u; /* Defines component Power value */


/*******************************************************************************
* Function Name: ADCBuffer_Init
********************************************************************************
*
* Summary:
*  Initializes or restores the component according to the customizer Configure 
*  dialog settings. It is not necessary to call Init() because the Start() API 
*  calls this function and is the preferred method to begin the component operation.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void ADCBuffer_Init(void)
{
    ADCBuffer_internalPower = ADCBuffer_POWER;
    ADCBuffer_CTB_CTRL_REG = ADCBuffer_DEFAULT_CTB_CTRL;
    ADCBuffer_OA_RES_CTRL_REG = ADCBuffer_DEFAULT_OA_RES_CTRL;
    ADCBuffer_OA_COMP_TRIM_REG = ADCBuffer_DEFAULT_OA_COMP_TRIM_REG;
}


/*******************************************************************************
* Function Name: ADCBuffer_Enable
********************************************************************************
*
* Summary:
*  Activates the hardware and begins the component operation. It is not necessary to 
*  call Enable() because the Start() API calls this function, which is the 
*  preferred method to begin the component operation.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void ADCBuffer_Enable(void)
{
    ADCBuffer_OA_RES_CTRL_REG |= ADCBuffer_internalPower | \
                                        ADCBuffer_OA_PUMP_EN;
}


/*******************************************************************************
* Function Name: ADCBuffer_Start
********************************************************************************
*
* Summary:
*  Performs all of the required initialization for the component and enables power 
*  to the block. The first time the routine is executed, the Power level, Mode, 
*  and Output mode are set. When called to restart the Opamp following a Stop() call, 
*  the current component parameter settings are retained.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  ADCBuffer_initVar: Used to check the initial configuration, modified
*  when this function is called for the first time.
*
*******************************************************************************/
void ADCBuffer_Start(void)
{
    if( 0u == ADCBuffer_initVar)
    {
        ADCBuffer_Init();
        ADCBuffer_initVar = 1u;
    }
    ADCBuffer_Enable();
}


/*******************************************************************************
* Function Name: ADCBuffer_Stop
********************************************************************************
*
* Summary:
*  Turn off the Opamp block.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void ADCBuffer_Stop(void)
{
    ADCBuffer_OA_RES_CTRL_REG &= ((uint32)~(ADCBuffer_OA_PWR_MODE_MASK | \
                                                   ADCBuffer_OA_PUMP_EN));
}


/*******************************************************************************
* Function Name: ADCBuffer_SetPower
********************************************************************************
*
* Summary:
*  Sets the Opamp to one of the three power levels.
*
* Parameters:
*  power: power levels.
*   ADCBuffer_LOW_POWER - Lowest active power
*   ADCBuffer_MED_POWER - Medium power
*   ADCBuffer_HIGH_POWER - Highest active power
*
* Return:
*  None
*
**********************************************************************************/
void ADCBuffer_SetPower(uint32 power)
{
    uint32 tmp;
    
    ADCBuffer_internalPower = ADCBuffer_GET_OA_PWR_MODE(power);
    tmp = ADCBuffer_OA_RES_CTRL_REG & \
           (uint32)~ADCBuffer_OA_PWR_MODE_MASK;
    ADCBuffer_OA_RES_CTRL_REG = tmp | ADCBuffer_internalPower;
}


/*******************************************************************************
* Function Name: ADCBuffer_PumpControl
********************************************************************************
*
* Summary:
*  Allows the user to turn the Opamp's boost pump on or off. By Default the Start() 
*  function turns on the pump. Use this API to turn it off. The boost must be 
*  turned on when the supply is less than 2.7 volts and off if the supply is more 
*  than 4 volts.
*
* Parameters:
*  onOff: Control the pump.
*   ADCBuffer_PUMP_OFF - Turn off the pump
*   ADCBuffer_PUMP_ON - Turn on the pump
*
* Return:
*  None
*
**********************************************************************************/
void ADCBuffer_PumpControl(uint32 onOff)
{
    
    if(0u != onOff)
    {
        ADCBuffer_OA_RES_CTRL |= ADCBuffer_OA_PUMP_EN;    
    }
    else
    {
        ADCBuffer_OA_RES_CTRL &= (uint32)~ADCBuffer_OA_PUMP_EN;
    }
}


/* [] END OF FILE */
