/*******************************************************************************
* File Name: ADCBuffer.h
* Version 1.10
*
* Description:
*  This file contains the function prototypes and constants used in
*  the Opamp (Analog Buffer) Component.
*
*
********************************************************************************
* Copyright 2013-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/


#if !defined(CY_OPAMP_ADCBuffer_H)
#define CY_OPAMP_ADCBuffer_H

#include "cytypes.h"
#include "cyfitter.h"


/***************************************
*       Type Definitions
***************************************/

/* Structure to save state before go to sleep */
typedef struct
{
    uint8  enableState;
} ADCBuffer_BACKUP_STRUCT;


/**************************************
*        Function Prototypes
**************************************/
void ADCBuffer_Init(void);
void ADCBuffer_Enable(void);
void ADCBuffer_Start(void);
void ADCBuffer_Stop(void);
void ADCBuffer_SetPower(uint32 power);
void ADCBuffer_PumpControl(uint32 onOff);
void ADCBuffer_Sleep(void);
void ADCBuffer_Wakeup(void);
void ADCBuffer_SaveConfig(void);
void ADCBuffer_RestoreConfig(void);


/**************************************
*           API Constants
**************************************/

/* Parameters for SetPower() function */
#define ADCBuffer_LOW_POWER      (1u)
#define ADCBuffer_MED_POWER      (2u)
#define ADCBuffer_HIGH_POWER     (3u)


/* Parameters for PumpControl() function */
#define ADCBuffer_PUMP_ON        (1u)
#define ADCBuffer_PUMP_OFF       (0u)


/***************************************
*   Initial Parameter Constants
****************************************/

#define ADCBuffer_OUTPUT_CURRENT         (1u)
#define ADCBuffer_POWER                  (1u)
#define ADCBuffer_MODE                   (1u)
#define ADCBuffer_OA_COMP_TRIM_VALUE     (2u)
#define ADCBuffer_DEEPSLEEP_SUPPORT      (0u)


/***************************************
*    Variables with External Linkage
***************************************/

extern uint8  ADCBuffer_initVar;


/**************************************
*             Registers
**************************************/
#define ADCBuffer_CTB_CTRL_REG       (*(reg32 *) ADCBuffer_cy_psoc4_abuf__CTBM_CTB_CTRL)
#define ADCBuffer_CTB_CTRL_PTR       ( (reg32 *) ADCBuffer_cy_psoc4_abuf__CTBM_CTB_CTRL)
#define ADCBuffer_OA_RES_CTRL_REG    (*(reg32 *) ADCBuffer_cy_psoc4_abuf__OA_RES_CTRL)
#define ADCBuffer_OA_RES_CTRL_PTR    ( (reg32 *) ADCBuffer_cy_psoc4_abuf__OA_RES_CTRL)
#define ADCBuffer_OA_COMP_TRIM_REG   (*(reg32 *) ADCBuffer_cy_psoc4_abuf__OA_COMP_TRIM)
#define ADCBuffer_OA_COMP_TRIM_PTR   ( (reg32 *) ADCBuffer_cy_psoc4_abuf__OA_COMP_TRIM)


/***************************************
*        Registers Constants
***************************************/

/* ADCBuffer_CTB_CTRL_REG */
#define ADCBuffer_CTB_CTRL_DEEPSLEEP_ON_SHIFT    (30u)   /* [30] Selects behavior CTB IP in the DeepSleep power mode */
#define ADCBuffer_CTB_CTRL_ENABLED_SHIFT         (31u)   /* [31] Enable of the CTB IP */


#define ADCBuffer_CTB_CTRL_DEEPSLEEP_ON          ((uint32) 0x01u << ADCBuffer_CTB_CTRL_DEEPSLEEP_ON_SHIFT)
#define ADCBuffer_CTB_CTRL_ENABLED               ((uint32) 0x01u << ADCBuffer_CTB_CTRL_ENABLED_SHIFT)


/* ADCBuffer_OA_RES_CTRL_REG */
#define ADCBuffer_OA_PWR_MODE_SHIFT          (0u)    /* [1:0]    Power level */
#define ADCBuffer_OA_DRIVE_STR_SEL_SHIFT     (2u)    /* [2]      Opamp output strenght select: 0 - 1x, 1 - 10x */
#define ADCBuffer_OA_COMP_EN_SHIFT           (4u)    /* [4]      CTB IP mode: 0 - Opamp, 1 - Comparator  */
#define ADCBuffer_OA_PUMP_EN_SHIFT           (11u)   /* [11]     Pump enable */


#define ADCBuffer_OA_PWR_MODE                ((uint32) 0x02u << ADCBuffer_OA_PWR_MODE_SHIFT)
#define ADCBuffer_OA_PWR_MODE_MASK           ((uint32) 0x03u << ADCBuffer_OA_PWR_MODE_SHIFT)
#define ADCBuffer_OA_DRIVE_STR_SEL_1X        ((uint32) 0x00u << ADCBuffer_OA_DRIVE_STR_SEL_SHIFT)
#define ADCBuffer_OA_DRIVE_STR_SEL_10X       ((uint32) 0x01u << ADCBuffer_OA_DRIVE_STR_SEL_SHIFT)
#define ADCBuffer_OA_DRIVE_STR_SEL_MASK      ((uint32) 0x01u << ADCBuffer_OA_DRIVE_STR_SEL_SHIFT)
#define ADCBuffer_OA_COMP_EN                 ((uint32) 0x00u << ADCBuffer_OA_COMP_EN_SHIFT)
#define ADCBuffer_OA_PUMP_EN                 ((uint32) 0x01u << ADCBuffer_OA_PUMP_EN_SHIFT)


/***************************************
*       Init Macros Definitions
***************************************/

#define ADCBuffer_GET_DEEPSLEEP_ON(deepSleep)    ((0u != (deepSleep)) ? (ADCBuffer_CTB_CTRL_DEEPSLEEP_ON) : (0u))
#define ADCBuffer_GET_OA_DRIVE_STR(current)      ((0u != (current)) ? (ADCBuffer_OA_DRIVE_STR_SEL_10X) : \
                                                                             (ADCBuffer_OA_DRIVE_STR_SEL_1X))
#define ADCBuffer_GET_OA_PWR_MODE(mode)          ((mode) & ADCBuffer_OA_PWR_MODE_MASK)
#define ADCBuffer_CHECK_PWR_MODE_OFF             (0u != (ADCBuffer_OA_RES_CTRL_REG & \
                                                                ADCBuffer_OA_PWR_MODE_MASK))

/* Returns true if component available in Deep Sleep power mode*/ 
#define ADCBuffer_CHECK_DEEPSLEEP_SUPPORT        (0u != ADCBuffer_DEEPSLEEP_SUPPORT) 

#define ADCBuffer_DEFAULT_CTB_CTRL (ADCBuffer_GET_DEEPSLEEP_ON(ADCBuffer_DEEPSLEEP_SUPPORT) | \
                                           ADCBuffer_CTB_CTRL_ENABLED)

#define ADCBuffer_DEFAULT_OA_RES_CTRL (ADCBuffer_OA_COMP_EN | \
                                              ADCBuffer_GET_OA_DRIVE_STR(ADCBuffer_OUTPUT_CURRENT))

#define ADCBuffer_DEFAULT_OA_COMP_TRIM_REG (ADCBuffer_OA_COMP_TRIM_VALUE)


/***************************************
* The following code is DEPRECATED and 
* should not be used in new projects.
***************************************/

#define ADCBuffer_LOWPOWER                   (ADCBuffer_LOW_POWER)
#define ADCBuffer_MEDPOWER                   (ADCBuffer_MED_POWER)
#define ADCBuffer_HIGHPOWER                  (ADCBuffer_HIGH_POWER)

/* PUMP ON/OFF defines */
#define ADCBuffer_PUMPON                     (ADCBuffer_PUMP_ON)
#define ADCBuffer_PUMPOFF                    (ADCBuffer_PUMP_OFF)

#define ADCBuffer_OA_CTRL                    (ADCBuffer_CTB_CTRL_REG)
#define ADCBuffer_OA_RES_CTRL                (ADCBuffer_OA_RES_CTRL_REG)

/* Bit Field  OA_CTRL */
#define ADCBuffer_OA_CTB_EN_SHIFT            (ADCBuffer_CTB_CTRL_ENABLED_SHIFT)
#define ADCBuffer_OA_PUMP_CTRL_SHIFT         (ADCBuffer_OA_PUMP_EN_SHIFT)
#define ADCBuffer_OA_PUMP_EN_MASK            (0x800u)
#define ADCBuffer_PUMP_PROTECT_MASK          (1u)


#endif    /* CY_OPAMP_ADCBuffer_H */


/* [] END OF FILE */
