/*******************************************************************************
* File Name: Buffer2.h
* Version 1.10
*
* Description:
*  This file contains the function prototypes and constants used in
*  the Opamp (Analog Buffer) Component.
*
*
********************************************************************************
* Copyright 2013-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/


#if !defined(CY_OPAMP_Buffer2_H)
#define CY_OPAMP_Buffer2_H

#include "cytypes.h"
#include "cyfitter.h"


/***************************************
*       Type Definitions
***************************************/

/* Structure to save state before go to sleep */
typedef struct
{
    uint8  enableState;
} Buffer2_BACKUP_STRUCT;


/**************************************
*        Function Prototypes
**************************************/
void Buffer2_Init(void);
void Buffer2_Enable(void);
void Buffer2_Start(void);
void Buffer2_Stop(void);
void Buffer2_SetPower(uint32 power);
void Buffer2_PumpControl(uint32 onOff);
void Buffer2_Sleep(void);
void Buffer2_Wakeup(void);
void Buffer2_SaveConfig(void);
void Buffer2_RestoreConfig(void);


/**************************************
*           API Constants
**************************************/

/* Parameters for SetPower() function */
#define Buffer2_LOW_POWER      (1u)
#define Buffer2_MED_POWER      (2u)
#define Buffer2_HIGH_POWER     (3u)


/* Parameters for PumpControl() function */
#define Buffer2_PUMP_ON        (1u)
#define Buffer2_PUMP_OFF       (0u)


/***************************************
*   Initial Parameter Constants
****************************************/

#define Buffer2_OUTPUT_CURRENT         (1u)
#define Buffer2_POWER                  (1u)
#define Buffer2_MODE                   (1u)
#define Buffer2_OA_COMP_TRIM_VALUE     (1u)
#define Buffer2_DEEPSLEEP_SUPPORT      (0u)


/***************************************
*    Variables with External Linkage
***************************************/

extern uint8  Buffer2_initVar;


/**************************************
*             Registers
**************************************/
#define Buffer2_CTB_CTRL_REG       (*(reg32 *) Buffer2_cy_psoc4_abuf__CTBM_CTB_CTRL)
#define Buffer2_CTB_CTRL_PTR       ( (reg32 *) Buffer2_cy_psoc4_abuf__CTBM_CTB_CTRL)
#define Buffer2_OA_RES_CTRL_REG    (*(reg32 *) Buffer2_cy_psoc4_abuf__OA_RES_CTRL)
#define Buffer2_OA_RES_CTRL_PTR    ( (reg32 *) Buffer2_cy_psoc4_abuf__OA_RES_CTRL)
#define Buffer2_OA_COMP_TRIM_REG   (*(reg32 *) Buffer2_cy_psoc4_abuf__OA_COMP_TRIM)
#define Buffer2_OA_COMP_TRIM_PTR   ( (reg32 *) Buffer2_cy_psoc4_abuf__OA_COMP_TRIM)


/***************************************
*        Registers Constants
***************************************/

/* Buffer2_CTB_CTRL_REG */
#define Buffer2_CTB_CTRL_DEEPSLEEP_ON_SHIFT    (30u)   /* [30] Selects behavior CTB IP in the DeepSleep power mode */
#define Buffer2_CTB_CTRL_ENABLED_SHIFT         (31u)   /* [31] Enable of the CTB IP */


#define Buffer2_CTB_CTRL_DEEPSLEEP_ON          ((uint32) 0x01u << Buffer2_CTB_CTRL_DEEPSLEEP_ON_SHIFT)
#define Buffer2_CTB_CTRL_ENABLED               ((uint32) 0x01u << Buffer2_CTB_CTRL_ENABLED_SHIFT)


/* Buffer2_OA_RES_CTRL_REG */
#define Buffer2_OA_PWR_MODE_SHIFT          (0u)    /* [1:0]    Power level */
#define Buffer2_OA_DRIVE_STR_SEL_SHIFT     (2u)    /* [2]      Opamp output strenght select: 0 - 1x, 1 - 10x */
#define Buffer2_OA_COMP_EN_SHIFT           (4u)    /* [4]      CTB IP mode: 0 - Opamp, 1 - Comparator  */
#define Buffer2_OA_PUMP_EN_SHIFT           (11u)   /* [11]     Pump enable */


#define Buffer2_OA_PWR_MODE                ((uint32) 0x02u << Buffer2_OA_PWR_MODE_SHIFT)
#define Buffer2_OA_PWR_MODE_MASK           ((uint32) 0x03u << Buffer2_OA_PWR_MODE_SHIFT)
#define Buffer2_OA_DRIVE_STR_SEL_1X        ((uint32) 0x00u << Buffer2_OA_DRIVE_STR_SEL_SHIFT)
#define Buffer2_OA_DRIVE_STR_SEL_10X       ((uint32) 0x01u << Buffer2_OA_DRIVE_STR_SEL_SHIFT)
#define Buffer2_OA_DRIVE_STR_SEL_MASK      ((uint32) 0x01u << Buffer2_OA_DRIVE_STR_SEL_SHIFT)
#define Buffer2_OA_COMP_EN                 ((uint32) 0x00u << Buffer2_OA_COMP_EN_SHIFT)
#define Buffer2_OA_PUMP_EN                 ((uint32) 0x01u << Buffer2_OA_PUMP_EN_SHIFT)


/***************************************
*       Init Macros Definitions
***************************************/

#define Buffer2_GET_DEEPSLEEP_ON(deepSleep)    ((0u != (deepSleep)) ? (Buffer2_CTB_CTRL_DEEPSLEEP_ON) : (0u))
#define Buffer2_GET_OA_DRIVE_STR(current)      ((0u != (current)) ? (Buffer2_OA_DRIVE_STR_SEL_10X) : \
                                                                             (Buffer2_OA_DRIVE_STR_SEL_1X))
#define Buffer2_GET_OA_PWR_MODE(mode)          ((mode) & Buffer2_OA_PWR_MODE_MASK)
#define Buffer2_CHECK_PWR_MODE_OFF             (0u != (Buffer2_OA_RES_CTRL_REG & \
                                                                Buffer2_OA_PWR_MODE_MASK))

/* Returns true if component available in Deep Sleep power mode*/ 
#define Buffer2_CHECK_DEEPSLEEP_SUPPORT        (0u != Buffer2_DEEPSLEEP_SUPPORT) 

#define Buffer2_DEFAULT_CTB_CTRL (Buffer2_GET_DEEPSLEEP_ON(Buffer2_DEEPSLEEP_SUPPORT) | \
                                           Buffer2_CTB_CTRL_ENABLED)

#define Buffer2_DEFAULT_OA_RES_CTRL (Buffer2_OA_COMP_EN | \
                                              Buffer2_GET_OA_DRIVE_STR(Buffer2_OUTPUT_CURRENT))

#define Buffer2_DEFAULT_OA_COMP_TRIM_REG (Buffer2_OA_COMP_TRIM_VALUE)


/***************************************
* The following code is DEPRECATED and 
* should not be used in new projects.
***************************************/

#define Buffer2_LOWPOWER                   (Buffer2_LOW_POWER)
#define Buffer2_MEDPOWER                   (Buffer2_MED_POWER)
#define Buffer2_HIGHPOWER                  (Buffer2_HIGH_POWER)

/* PUMP ON/OFF defines */
#define Buffer2_PUMPON                     (Buffer2_PUMP_ON)
#define Buffer2_PUMPOFF                    (Buffer2_PUMP_OFF)

#define Buffer2_OA_CTRL                    (Buffer2_CTB_CTRL_REG)
#define Buffer2_OA_RES_CTRL                (Buffer2_OA_RES_CTRL_REG)

/* Bit Field  OA_CTRL */
#define Buffer2_OA_CTB_EN_SHIFT            (Buffer2_CTB_CTRL_ENABLED_SHIFT)
#define Buffer2_OA_PUMP_CTRL_SHIFT         (Buffer2_OA_PUMP_EN_SHIFT)
#define Buffer2_OA_PUMP_EN_MASK            (0x800u)
#define Buffer2_PUMP_PROTECT_MASK          (1u)


#endif    /* CY_OPAMP_Buffer2_H */


/* [] END OF FILE */
