/*******************************************************************************
* File Name: PosBias.c
* Version 1.20
*
* Description:
*  This file provides the source code to the API for the Opamp (Analog Buffer)
*  Component.
*
*
********************************************************************************
* Copyright 2013-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "PosBias.h"

uint8 PosBias_initVar = 0u; /* Defines if component was initialized */
static uint32 PosBias_internalPower = 0u; /* Defines component Power value */


/*******************************************************************************
* Function Name: PosBias_Init
********************************************************************************
*
* Summary:
*  Initializes or restores the component according to the customizer Configure 
*  dialog settings. It is not necessary to call Init() because the Start() API 
*  calls this function and is the preferred method to begin the component operation.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void PosBias_Init(void)
{
    PosBias_internalPower = PosBias_POWER;
    PosBias_CTB_CTRL_REG = PosBias_DEFAULT_CTB_CTRL;
    PosBias_OA_RES_CTRL_REG = PosBias_DEFAULT_OA_RES_CTRL;
    PosBias_OA_COMP_TRIM_REG = PosBias_DEFAULT_OA_COMP_TRIM_REG;
}


/*******************************************************************************
* Function Name: PosBias_Enable
********************************************************************************
*
* Summary:
*  Activates the hardware and begins the component operation. It is not necessary to 
*  call Enable() because the Start() API calls this function, which is the 
*  preferred method to begin the component operation.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void PosBias_Enable(void)
{
    PosBias_OA_RES_CTRL_REG |= PosBias_internalPower | \
                                        PosBias_OA_PUMP_EN;
}


/*******************************************************************************
* Function Name: PosBias_Start
********************************************************************************
*
* Summary:
*  Performs all of the required initialization for the component and enables power 
*  to the block. The first time the routine is executed, the Power level, Mode, 
*  and Output mode are set. When called to restart the Opamp following a Stop() call, 
*  the current component parameter settings are retained.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  PosBias_initVar: Used to check the initial configuration, modified
*  when this function is called for the first time.
*
*******************************************************************************/
void PosBias_Start(void)
{
    if( 0u == PosBias_initVar)
    {
        PosBias_Init();
        PosBias_initVar = 1u;
    }
    PosBias_Enable();
}


/*******************************************************************************
* Function Name: PosBias_Stop
********************************************************************************
*
* Summary:
*  Turn off the Opamp block.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void PosBias_Stop(void)
{
    PosBias_OA_RES_CTRL_REG &= ((uint32)~(PosBias_OA_PWR_MODE_MASK | \
                                                   PosBias_OA_PUMP_EN));
}


/*******************************************************************************
* Function Name: PosBias_SetPower
********************************************************************************
*
* Summary:
*  Sets the Opamp to one of the three power levels.
*
* Parameters:
*  power: power levels.
*   PosBias_LOW_POWER - Lowest active power
*   PosBias_MED_POWER - Medium power
*   PosBias_HIGH_POWER - Highest active power
*
* Return:
*  None
*
**********************************************************************************/
void PosBias_SetPower(uint32 power)
{
    uint32 tmp;
    
    PosBias_internalPower = PosBias_GET_OA_PWR_MODE(power);
    tmp = PosBias_OA_RES_CTRL_REG & \
           (uint32)~PosBias_OA_PWR_MODE_MASK;
    PosBias_OA_RES_CTRL_REG = tmp | PosBias_internalPower;
}


/*******************************************************************************
* Function Name: PosBias_PumpControl
********************************************************************************
*
* Summary:
*  Allows the user to turn the Opamp's boost pump on or off. By Default the Start() 
*  function turns on the pump. Use this API to turn it off. The boost must be 
*  turned on when the supply is less than 2.7 volts and off if the supply is more 
*  than 4 volts.
*
* Parameters:
*  onOff: Control the pump.
*   PosBias_PUMP_OFF - Turn off the pump
*   PosBias_PUMP_ON - Turn on the pump
*
* Return:
*  None
*
**********************************************************************************/
void PosBias_PumpControl(uint32 onOff)
{
    
    if(0u != onOff)
    {
        PosBias_OA_RES_CTRL |= PosBias_OA_PUMP_EN;    
    }
    else
    {
        PosBias_OA_RES_CTRL &= (uint32)~PosBias_OA_PUMP_EN;
    }
}


/* [] END OF FILE */
