/*******************************************************************************
* File Name: PosBias.h
* Version 1.20
*
* Description:
*  This file contains the function prototypes and constants used in
*  the Opamp (Analog Buffer) Component.
*
*
********************************************************************************
* Copyright 2013-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/


#if !defined(CY_OPAMP_PosBias_H)
#define CY_OPAMP_PosBias_H

#include "cytypes.h"
#include "cyfitter.h"


/***************************************
*       Type Definitions
***************************************/

/* Structure to save state before go to sleep */
typedef struct
{
    uint8  enableState;
} PosBias_BACKUP_STRUCT;


/**************************************
*        Function Prototypes
**************************************/
void PosBias_Init(void);
void PosBias_Enable(void);
void PosBias_Start(void);
void PosBias_Stop(void);
void PosBias_SetPower(uint32 power);
void PosBias_PumpControl(uint32 onOff);
void PosBias_Sleep(void);
void PosBias_Wakeup(void);
void PosBias_SaveConfig(void);
void PosBias_RestoreConfig(void);


/**************************************
*           API Constants
**************************************/

/* Parameters for SetPower() function */
#define PosBias_LOW_POWER      (1u)
#define PosBias_MED_POWER      (2u)
#define PosBias_HIGH_POWER     (3u)


/* Parameters for PumpControl() function */
#define PosBias_PUMP_ON        (1u)
#define PosBias_PUMP_OFF       (0u)


/***************************************
*   Initial Parameter Constants
****************************************/

#define PosBias_OUTPUT_CURRENT         (1u)
#define PosBias_POWER                  (1u)
#define PosBias_MODE                   (0u)
#define PosBias_OA_COMP_TRIM_VALUE     (3u)
#define PosBias_DEEPSLEEP_SUPPORT      (0u)


/***************************************
*    Variables with External Linkage
***************************************/

extern uint8  PosBias_initVar;


/**************************************
*             Registers
**************************************/

#ifdef CYIPBLOCK_m0s8pass4b_VERSION
    #define PosBias_CTB_CTRL_REG       (*(reg32 *) PosBias_cy_psoc4_abuf__CTB_CTB_CTRL)
    #define PosBias_CTB_CTRL_PTR       ( (reg32 *) PosBias_cy_psoc4_abuf__CTB_CTB_CTRL)
#else
    #define PosBias_CTB_CTRL_REG       (*(reg32 *) PosBias_cy_psoc4_abuf__CTBM_CTB_CTRL)
    #define PosBias_CTB_CTRL_PTR       ( (reg32 *) PosBias_cy_psoc4_abuf__CTBM_CTB_CTRL)
#endif /* CYIPBLOCK_m0s8pass4b_VERSION */

#define PosBias_OA_RES_CTRL_REG    (*(reg32 *) PosBias_cy_psoc4_abuf__OA_RES_CTRL)
#define PosBias_OA_RES_CTRL_PTR    ( (reg32 *) PosBias_cy_psoc4_abuf__OA_RES_CTRL)
#define PosBias_OA_COMP_TRIM_REG   (*(reg32 *) PosBias_cy_psoc4_abuf__OA_COMP_TRIM)
#define PosBias_OA_COMP_TRIM_PTR   ( (reg32 *) PosBias_cy_psoc4_abuf__OA_COMP_TRIM)


/***************************************
*        Registers Constants
***************************************/

/* PosBias_CTB_CTRL_REG */
#define PosBias_CTB_CTRL_DEEPSLEEP_ON_SHIFT    (30u)   /* [30] Selects behavior CTB IP in the DeepSleep power mode */
#define PosBias_CTB_CTRL_ENABLED_SHIFT         (31u)   /* [31] Enable of the CTB IP */


#define PosBias_CTB_CTRL_DEEPSLEEP_ON          ((uint32) 0x01u << PosBias_CTB_CTRL_DEEPSLEEP_ON_SHIFT)
#define PosBias_CTB_CTRL_ENABLED               ((uint32) 0x01u << PosBias_CTB_CTRL_ENABLED_SHIFT)


/* PosBias_OA_RES_CTRL_REG */
#define PosBias_OA_PWR_MODE_SHIFT          (0u)    /* [1:0]    Power level */
#define PosBias_OA_DRIVE_STR_SEL_SHIFT     (2u)    /* [2]      Opamp output strenght select: 0 - 1x, 1 - 10x */
#define PosBias_OA_COMP_EN_SHIFT           (4u)    /* [4]      CTB IP mode: 0 - Opamp, 1 - Comparator  */
#define PosBias_OA_PUMP_EN_SHIFT           (11u)   /* [11]     Pump enable */


#define PosBias_OA_PWR_MODE                ((uint32) 0x02u << PosBias_OA_PWR_MODE_SHIFT)
#define PosBias_OA_PWR_MODE_MASK           ((uint32) 0x03u << PosBias_OA_PWR_MODE_SHIFT)
#define PosBias_OA_DRIVE_STR_SEL_1X        ((uint32) 0x00u << PosBias_OA_DRIVE_STR_SEL_SHIFT)
#define PosBias_OA_DRIVE_STR_SEL_10X       ((uint32) 0x01u << PosBias_OA_DRIVE_STR_SEL_SHIFT)
#define PosBias_OA_DRIVE_STR_SEL_MASK      ((uint32) 0x01u << PosBias_OA_DRIVE_STR_SEL_SHIFT)
#define PosBias_OA_COMP_EN                 ((uint32) 0x00u << PosBias_OA_COMP_EN_SHIFT)
#define PosBias_OA_PUMP_EN                 ((uint32) 0x01u << PosBias_OA_PUMP_EN_SHIFT)


/***************************************
*       Init Macros Definitions
***************************************/

#define PosBias_GET_DEEPSLEEP_ON(deepSleep)    ((0u != (deepSleep)) ? (PosBias_CTB_CTRL_DEEPSLEEP_ON) : (0u))
#define PosBias_GET_OA_DRIVE_STR(current)      ((0u != (current)) ? (PosBias_OA_DRIVE_STR_SEL_10X) : \
                                                                             (PosBias_OA_DRIVE_STR_SEL_1X))
#define PosBias_GET_OA_PWR_MODE(mode)          ((mode) & PosBias_OA_PWR_MODE_MASK)
#define PosBias_CHECK_PWR_MODE_OFF             (0u != (PosBias_OA_RES_CTRL_REG & \
                                                                PosBias_OA_PWR_MODE_MASK))

/* Returns true if component available in Deep Sleep power mode*/ 
#define PosBias_CHECK_DEEPSLEEP_SUPPORT        (0u != PosBias_DEEPSLEEP_SUPPORT) 

#define PosBias_DEFAULT_CTB_CTRL (PosBias_GET_DEEPSLEEP_ON(PosBias_DEEPSLEEP_SUPPORT) | \
                                           PosBias_CTB_CTRL_ENABLED)

#define PosBias_DEFAULT_OA_RES_CTRL (PosBias_OA_COMP_EN | \
                                              PosBias_GET_OA_DRIVE_STR(PosBias_OUTPUT_CURRENT))

#define PosBias_DEFAULT_OA_COMP_TRIM_REG (PosBias_OA_COMP_TRIM_VALUE)


/***************************************
* The following code is DEPRECATED and 
* should not be used in new projects.
***************************************/

#define PosBias_LOWPOWER                   (PosBias_LOW_POWER)
#define PosBias_MEDPOWER                   (PosBias_MED_POWER)
#define PosBias_HIGHPOWER                  (PosBias_HIGH_POWER)

/* PUMP ON/OFF defines */
#define PosBias_PUMPON                     (PosBias_PUMP_ON)
#define PosBias_PUMPOFF                    (PosBias_PUMP_OFF)

#define PosBias_OA_CTRL                    (PosBias_CTB_CTRL_REG)
#define PosBias_OA_RES_CTRL                (PosBias_OA_RES_CTRL_REG)

/* Bit Field  OA_CTRL */
#define PosBias_OA_CTB_EN_SHIFT            (PosBias_CTB_CTRL_ENABLED_SHIFT)
#define PosBias_OA_PUMP_CTRL_SHIFT         (PosBias_OA_PUMP_EN_SHIFT)
#define PosBias_OA_PUMP_EN_MASK            (0x800u)
#define PosBias_PUMP_PROTECT_MASK          (1u)


#endif    /* CY_OPAMP_PosBias_H */


/* [] END OF FILE */
