/*******************************************************************************
* File Name: PowerStat32.c  
* Version 2.20
*
* Description:
*  This file contains APIs to set up the Pins component for low power modes.
*
* Note:
*
********************************************************************************
* Copyright 2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "PowerStat32.h"

static PowerStat32_BACKUP_STRUCT  PowerStat32_backup = {0u, 0u, 0u};


/*******************************************************************************
* Function Name: PowerStat32_Sleep
****************************************************************************//**
*
* \brief Stores the pin configuration and prepares the pin for entering chip 
*  deep-sleep/hibernate modes. This function must be called for SIO and USBIO
*  pins. It is not essential if using GPIO or GPIO_OVT pins.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None 
*  
* \sideeffect
*  For SIO pins, this function configures the pin input threshold to CMOS and
*  drive level to Vddio. This is needed for SIO pins when in device 
*  deep-sleep/hibernate modes.
*
* \funcusage
*  \snippet PowerStat32_SUT.c usage_PowerStat32_Sleep_Wakeup
*******************************************************************************/
void PowerStat32_Sleep(void)
{
    #if defined(PowerStat32__PC)
        PowerStat32_backup.pcState = PowerStat32_PC;
    #else
        #if (CY_PSOC4_4200L)
            /* Save the regulator state and put the PHY into suspend mode */
            PowerStat32_backup.usbState = PowerStat32_CR1_REG;
            PowerStat32_USB_POWER_REG |= PowerStat32_USBIO_ENTER_SLEEP;
            PowerStat32_CR1_REG &= PowerStat32_USBIO_CR1_OFF;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(PowerStat32__SIO)
        PowerStat32_backup.sioState = PowerStat32_SIO_REG;
        /* SIO requires unregulated output buffer and single ended input buffer */
        PowerStat32_SIO_REG &= (uint32)(~PowerStat32_SIO_LPM_MASK);
    #endif  
}


/*******************************************************************************
* Function Name: PowerStat32_Wakeup
****************************************************************************//**
*
* \brief Restores the pin configuration that was saved during Pin_Sleep().
*
* For USBIO pins, the wakeup is only triggered for falling edge interrupts.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None
*  
* \funcusage
*  Refer to PowerStat32_Sleep() for an example usage.
*******************************************************************************/
void PowerStat32_Wakeup(void)
{
    #if defined(PowerStat32__PC)
        PowerStat32_PC = PowerStat32_backup.pcState;
    #else
        #if (CY_PSOC4_4200L)
            /* Restore the regulator state and come out of suspend mode */
            PowerStat32_USB_POWER_REG &= PowerStat32_USBIO_EXIT_SLEEP_PH1;
            PowerStat32_CR1_REG = PowerStat32_backup.usbState;
            PowerStat32_USB_POWER_REG &= PowerStat32_USBIO_EXIT_SLEEP_PH2;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(PowerStat32__SIO)
        PowerStat32_SIO_REG = PowerStat32_backup.sioState;
    #endif
}


/* [] END OF FILE */
