/*************************************************** 
 This is a library for the MCP23017 i2c port expander

 These displays use I2C to communicate, 2 pins are required to
 interface
 Adafruit invests time and resources providing this open source code,
 please support Adafruit and open-source hardware by purchasing
 products from Adafruit!

 ****************************************************/

#include "MCP23017.h"

// Bit number associated to a give Pin
uint8_t bitForPin(uint8_t pin)
{
	return pin%8;
}

// Register address, port dependent, for a given PIN
uint8_t regForPin(uint8_t pin, uint8_t portAaddr, uint8_t portBaddr)
{
	return(pin<8) ?portAaddr:portBaddr;
}

// Reads an individual register
uint8_t readRegister(uint8_t addr, uint8_t regAddr)
{
	// read the current GPINTEN   
	return read8(MCP23017_ADDRESS | addr, regAddr);
}

// Writes and individual register
void writeRegister(uint8_t addr, uint8_t regAddr, uint8_t regValue)
{
    write8(MCP23017_ADDRESS | addr, regAddr, regValue);
}

/**
 * Helper to update a single bit of an A/B register.
 * - Reads the current register value
 * - Writes the new register value
 */
void updateRegisterBit(uint8_t addr, uint8_t pin, uint8_t pValue, uint8_t portAaddr, uint8_t portBaddr)
{
	uint8_t regValue;
	uint8_t regAddr=regForPin(pin,portAaddr,portBaddr);
	uint8_t bit=bitForPin(pin);
	regValue = readRegister(addr, regAddr);

	// set the value for the particular bit
	bitWrite(regValue,bit,pValue);
	writeRegister(addr, regAddr,regValue);
}


// Initializes the MCP23017 given its HW selected address, see datasheet for Address selection.
void MCP23017_begin(uint8_t addr)
{
	if (addr > 7)
        addr = 7;

	// set defaults!
	// all inputs on port A and B
	writeRegister(addr,MCP23017_IODIRA,0xff);
	writeRegister(addr,MCP23017_IODIRB,0xff);  
}

// Sets the pin mode to either INPUT or OUTPUT
void pinMode(uint8_t addr, uint8_t p, uint8_t d) 
{
	updateRegisterBit(addr,p,(d==INPUT),MCP23017_IODIRA,MCP23017_IODIRB);
}

// Reads all 16 pins (port A and B) into a single 16 bits variable.
uint16_t readGPIOAB(uint8_t addr)
{
	return read16(MCP23017_ADDRESS | addr,MCP23017_GPIOA);
}

/**
 * Read a single port, A or B, and return its current 8 bit value.
 * Parameter b should be 0 for GPIOA, and 1 for GPIOB.
 */
uint8_t readGPIO(uint8_t addr, uint8_t b) 
{
	// read the current GPIO output latches
    if (b == 0)
        return read8(MCP23017_ADDRESS | addr,MCP23017_GPIOA);
    else
        return read8(MCP23017_ADDRESS | addr,MCP23017_GPIOB);
}

 // Writes all the pins in one go. This method is very useful if you are implementing 
 // a multiplexed matrix and want to get a decent refresh rate.
void writeGPIOAB(uint8_t addr, uint16_t ba) 
{
    write16(MCP23017_ADDRESS | addr,MCP23017_GPIOA,ba);
}

// Writes a value to a digital pin
void pinWrite(uint8_t addr, uint8_t pin, uint8_t d) 
{
	uint8_t gpio;
	uint8_t bit = bitForPin(pin);

	// read the current GPIO output latches
	uint8_t regAddr = regForPin(pin,MCP23017_OLATA,MCP23017_OLATB);
	gpio = readRegister(addr, regAddr);

	// set the pin and direction
	bitWrite(gpio,bit,d);

	// write the new GPIO
	regAddr = regForPin(pin,MCP23017_GPIOA,MCP23017_GPIOB);
	writeRegister(addr, regAddr,gpio);
}

void pullUp(uint8_t addr, uint8_t p, uint8_t d) 
{
	updateRegisterBit(addr,p,d,MCP23017_GPPUA,MCP23017_GPPUB);
}

uint8_t digitalRead(uint8_t addr, uint8_t pin) 
{
	uint8_t bit = bitForPin(pin);
	uint8_t regAddr = regForPin(pin,MCP23017_GPIOA,MCP23017_GPIOB);
	return (readRegister(addr,regAddr) >> bit) & 0x1;
}

/**
 * Configures the interrupt system. both port A and B are assigned the same configuration.
 * Mirroring will OR both INTA and INTB pins.
 * Opendrain will set the INT pin to value or open drain.
 * polarity will set LOW or HIGH on interrupt.
 * Default values after Power On Reset are: (false,flase, LOW)
 * If you are connecting the INTA/B pin to arduino 2/3, you should configure the interupt handling as FALLING with
 * the default configuration.
 */
void setupInterrupts(uint8_t addr, uint8_t mirroring, uint8_t openDrain, uint8_t polarity){
	// configure the port A
	uint8_t ioconfValue=readRegister(addr,MCP23017_IOCONA);
	bitWrite(ioconfValue,6,mirroring);
	bitWrite(ioconfValue,2,openDrain);
	bitWrite(ioconfValue,1,polarity);
	writeRegister(addr,MCP23017_IOCONA,ioconfValue);

	// Configure the port B
	ioconfValue=readRegister(addr,MCP23017_IOCONB);
	bitWrite(ioconfValue,6,mirroring);
	bitWrite(ioconfValue,2,openDrain);
	bitWrite(ioconfValue,1,polarity);
	writeRegister(addr,MCP23017_IOCONB,ioconfValue);
}

/**
 * Set's up a pin for interrupt. uses arduino MODEs: CHANGE, FALLING, RISING.
 *
 * Note that the interrupt condition finishes when you read the information about the port / value
 * that caused the interrupt or you read the port itself. Check the datasheet can be confusing.
 *
 */
void setupInterruptPin(uint8_t addr, uint8_t pin, uint8_t mode) 
{
	// set the pin interrupt control (0 means change, 1 means compare against given value);
	updateRegisterBit(addr,pin,(mode!=CHANGE),MCP23017_INTCONA,MCP23017_INTCONB);
	// if the mode is not CHANGE, we need to set up a default value, different value triggers interrupt

	// In a RISING interrupt the default value is 0, interrupt is triggered when the pin goes to 1.
	// In a FALLING interrupt the default value is 1, interrupt is triggered when pin goes to 0.
	updateRegisterBit(addr,pin,(mode==FALLING),MCP23017_DEFVALA,MCP23017_DEFVALB);

	// enable the pin for interrupt
	updateRegisterBit(addr,pin,HIGH,MCP23017_GPINTENA,MCP23017_GPINTENB);
}

uint8_t getLastInterruptPin(uint8_t addr)
{
	uint8_t intf; uint8_t i;

	// try port A
	intf=readRegister(addr,MCP23017_INTFA);
	for(i=0;i<8;i++) if (bitRead(intf,i)) return i;

	// try port B
	intf=readRegister(addr,MCP23017_INTFB);
	for(i=0;i<8;i++) if (bitRead(intf,i)) return i+8;

	return MCP23017_INT_ERR;

}

uint8_t getLastInterruptPinValue(uint8_t addr)
{
	uint8_t intPin=getLastInterruptPin(addr);
	if(intPin!=MCP23017_INT_ERR){
		uint8_t intcapreg=regForPin(intPin,MCP23017_INTCAPA,MCP23017_INTCAPB);
		uint8_t bit=bitForPin(intPin);
		return (readRegister(addr,intcapreg)>>bit) & (0x01);
	}

	return MCP23017_INT_ERR;
}


